package edu.ksu.cis.ndpc;

import java.util.HashSet;

/**
 * @author Brian Mulanda
 * 
 */
public class PropLSymbols extends LangSymbols {
	private HashSet pSymbols;
	private HashSet sSymbols;

	public PropLSymbols() {
		pSymbols = new HashSet();
		sSymbols = new HashSet();

		sSymbols.add("(");
		sSymbols.add(")");
		sSymbols.add("v");
		sSymbols.add("V");
		sSymbols.add("^");
		sSymbols.add("~");
		sSymbols.add("->");
		sSymbols.add("<->");
		sSymbols.add("_|_");

		pSymbols.add("p");
		pSymbols.add("q");
		pSymbols.add("r");
		pSymbols.add("s");
		pSymbols.add("t");
		pSymbols.add("c");
		pSymbols.add("n");
		pSymbols.add("h");
		pSymbols.add("l");
	}

	public boolean isSpecial(String symbol) {
		boolean result = false;
		result = sSymbols.contains(symbol);
		return result;
	}

	public int identify(String symbol) {
		int type = S_INV;
		if (symbol != null) {
			symbol = symbol.trim();
			if (symbol.equals("(")) {
				type = S_OPAR;
			} else if (symbol.equals(")")) {
				type = S_CPAR;
			} else if (symbol.equalsIgnoreCase("v")) {
				type = S_DISJ;
			} else if (symbol.equals("^")) {
				type = S_CONJ;
			} else if (symbol.equals("~")) {
				type = S_NEG;
			} else if (symbol.equals("->")) {
				type = S_IMPL;
			} else if (symbol.equals("<->")) {
				type = S_DIMPL;
			} else if (symbol.equals("_|_")) {
				type = S_BOTTOM;
			} else if (pSymbols.contains(symbol.substring(0, 1))) {
				type = S_ATOM;
			}
		}
		return type;
	}

	public boolean isAtom(int type) {
		return type == S_ATOM;
	}

	public boolean isConnective(int type) {
		return (type == S_DISJ) || (type == S_CONJ) || (type == S_IMPL)
				|| (type == S_DIMPL);
	}

	public boolean isConnective(Symbol symbol) {
		boolean result = false;
		if (symbol != null) {
			result = isConnective(symbol.getType());
		}
		return result;
	}

	public boolean isOperator(Symbol symbol) {
		if (symbol != null) {
			return isOperator(symbol.getType());
		} else {
			return false;
		}
	}

	public boolean isOperator(int type) {
		boolean retval = false;
		retval = (type == S_DISJ) || (type == S_CONJ) || (type == S_IMPL)
				|| (type == S_DIMPL) || (type == S_NEG);
		return retval;
	}

	public String[] getSpecialSymbols() {
		return (String[]) sSymbols.toArray(new String[sSymbols.size()]);
	}

	public int getAssociativity(int opSymbolType) {
		return ASSOC_RIGHT;
	}

	public int getRelativePrecedence(int opSymbol1Type, int opSymbol2Type) {
		int precedence = PRECE_EQUAL;
		if (opSymbol1Type == opSymbol2Type) {
			int assoc = getAssociativity(opSymbol1Type);
			switch (assoc) {
			case ASSOC_LEFT:
				precedence = PRECE_HIGHER;
				break;
			case ASSOC_RIGHT:
				precedence = PRECE_LOWER;
				break;
			}
		} else {
			if (getPrecedence(opSymbol1Type) < getPrecedence(opSymbol2Type)) {
				precedence = PRECE_HIGHER;
			} else if (getPrecedence(opSymbol1Type) > getPrecedence(opSymbol2Type)) {
				precedence = PRECE_LOWER;
			} else {
				precedence = PRECE_EQUAL;
			}
			// System.out.println("check precedence: " + precedence);
		}
		return precedence;
	}

	public int getPrecedence(int opType) {
		int precedence = Integer.MAX_VALUE;
		if (opType != S_NULL) {
			if (opType == S_NEG) {
				precedence = 1;
			} else if (opType == S_CONJ) {
				precedence = 2;
			} else if (opType == S_DISJ) {
				precedence = 2;
			} else if (opType == S_IMPL) {
				precedence = 3;
			} else if (opType == S_DIMPL) {
				precedence = 3;
			}
		}
		return precedence;
	}

	public boolean exists(String symbol) {
		return false;
	}

	public void forget(String symbol) {
	}

}