package edu.ksu.cis.ndpc.gui;

import java.awt.Graphics;
import java.awt.FontMetrics;

public class Drawer {
	private int charWidth;
	private int charHeight;
	private char[] drawChars;

	public Drawer(int charWidth, int charHeight) {
		this.charWidth = charWidth;
		this.charHeight = charHeight;
		this.drawChars = new char[50];
	}

	public int getCharWidth() {
		return charWidth;
	}

	public int getCharHeight() {
		return charHeight;
	}

	public int draw(Graphics g, char[] c, int x, int y, boolean check) {
		return draw(g, c, 0, c.length, x, y, check);
	}

	public int draw(Graphics g, char[] c, int offset, int length, int x, int y,
			boolean check) {
		int advance = 0;

		// assert(c != null && c.length > 0)
		// assert(offset >= 0 && length >= 0)
		// assert(offset + length < c.length)

		if (check) {
			for (int i = 0; i < c.length; ++i) {
				advance += draw(g, c[i], x + advance, y, true);
			}
		} else {
			g.drawChars(c, offset, length, x, y);
			FontMetrics fm = g.getFontMetrics();
			if (charWidth > 0) {
				advance = charWidth * c.length;
			} else {
				for (int i = 0; i < c.length; ++i) {
					advance += fm.charWidth(c[i]);
				}
			}
		}

		return advance;
	}

	public int draw(Graphics g, char c, int x, int y, boolean check) {
		int advance = 0;
		if (check && isSpecialChar(c)) {
			advance = drawSpecialChar(g, c, x, y);
		} else {
			advance = draw(g, c, x, y);
		}
		return advance;
	}

	public int draw(Graphics g, char c, int x, int y) {
		int advance = 0;

		// assert(c is printable)
		drawChars[0] = c;
		g.drawChars(drawChars, 0, 1, x, y);
		if (charWidth > 0) {
			advance = charWidth;
		} else {
			advance = g.getFontMetrics().charWidth(c);
		}
		return advance;
	}

	boolean isSpecialChar(char c) {
		return (c == 'A' || c == 'E' || c == '_');
	}

	public int drawSpecialChar(Graphics g, char c, int x, int y) {
		if (c == 'A') {
			x += 1;
			y -= 1;
			int cwidth = charWidth - 2;
			cwidth = cwidth % 2 == 0 ? cwidth - 1 : cwidth;
			int cheight = charHeight - 1;
			cheight = cheight % 2 == 0 ? cheight - 1 : cheight;

			g.drawLine(x, y - cheight, x + (cwidth / 2), y); // left diagonal
			g.drawLine(x + cwidth, y - cheight, x + (cwidth / 2), y); // right
																		// diagonal
			g.drawLine(x + (cwidth / 4), y - (cheight / 2) - 2, x
					+ (3 * (cwidth / 4)), y - (cheight / 2) - 2); // horizontal
																	// bar
		} else if (c == 'E') {
			x += 1;
			y -= 1;
			int cwidth = charWidth - 2;
			cwidth = cwidth % 2 == 0 ? cwidth - 1 : cwidth;
			int cheight = charHeight - 1;
			cheight = cheight % 2 == 0 ? cheight - 1 : cheight;

			g.drawLine(x + cwidth, y - cheight, x + cwidth, y); // vertical
			g.drawLine(x, y - cheight, x + cwidth, y - cheight); // horizontal
																	// top
			g.drawLine(x, y, x + cwidth, y); // horizontal middle
			g.drawLine(x + (cwidth / 3), y - (cheight / 2), x + cwidth, y
					- (cheight / 2)); // horizontal middle
		} else if (c == '_') {
			x += 1;
			y -= 1;

			int cwidth = charWidth - 2;
			cwidth = cwidth % 2 == 0 ? cwidth - 1 : cwidth;
			int cheight = charHeight - 1;

			g.drawLine(x, y, x + cwidth, y); // horizontal base
			g.drawLine(x + (cwidth / 2), y, x + (cwidth / 2), y
					- ((2 * cheight) / 3)); // vertical middle
			/*
			 * //y = y - 2; //g.drawLine(x, y, x + (charWidth2) - 1, y);
			 * g.drawLine(x, y, x + charWidth, y); //g.drawLine(x + charWidth -
			 * 1, y, x + charWidth - 1, y - (charWidth + (charWidth/2)));
			 * g.drawLine(x + (charWidth/2), y, x + (charWidth/2), y -
			 * (charWidth + (charWidth/4))); //advance = (fm.charWidth(c) 2);
			 */

		}
		return charWidth;
	}
}
