/**
 * 
 * @author Brian Mulanda
 * 
 */

package edu.ksu.cis.ndpc.gui;

import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;

public class GUIListener implements KeyListener, MouseListener,
		MouseMotionListener {

	private GUIHandler guiHandler;
	private boolean dragging;
	// private int dragX;
	// private int dragY;
	private int mouseButton;

	public GUIListener(GUIHandler handler) {
		this.guiHandler = handler;
		// System.out.println("Button1 = " + MouseEvent.BUTTON1);
		// System.out.println("Button2 = " + MouseEvent.BUTTON2);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 * 
	 *      Invoked when a key has been pressed.
	 */
	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
			if (dragging) {
				dragging = false;
				guiHandler.cancel();
			}
		}
		guiHandler.pressKey(e);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 * 
	 *      Invoked when a key has been released.
	 */
	public void keyReleased(KeyEvent e) { /* Not interested */
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent) Invoked
	 *      when a key has been typed.
	 */
	public void keyTyped(KeyEvent e) { /* Not interested */
	}

	/**
	 * (non-Javadoc)
	 * 
	 * Invoked when a mouse button is pressed on a component and then dragged.
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */

	public void mouseDragged(MouseEvent e) {
		// System.out.println("Drag mouse: " + e.getButton());
		if (mouseButton == MouseEvent.BUTTON1) {
			if (!dragging) {
				guiHandler.beginDragMouse(e.getPoint().x, e.getPoint().y);
				dragging = true;
			} else {
				guiHandler.dragMouse(e.getPoint().x, e.getPoint().y);
			}
		}
		// Ignore right-mouse button
	}

	/**
	 * (non-Javadoc)
	 * 
	 * Invoked when the mouse cursor has been moved onto a component but no
	 * buttons have been pushed.
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e) {
		guiHandler.moveMouse(e.getPoint().x, e.getPoint().y);
	}

	/**
	 * (non-Javadoc)
	 * 
	 * Invoked when the mouse button has been clicked (pressed and released) on
	 * a component.
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
		if (e.getButton() == MouseEvent.BUTTON1) {
			guiHandler.clickMouse(e.getPoint().x, e.getPoint().y);
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * Invoked when the mouse enters a component.
	 * 
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) { /* Not Interested */
	}

	/**
	 * (non-Javadoc)
	 * 
	 * Invoked when the mouse exits a component.
	 * 
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) { /* Not Interested */
	}

	/**
	 * (non-Javadoc)
	 * 
	 * Invoked when a mouse button has been pressed on a component.
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		mouseButton = e.getButton();
		if (mouseButton == MouseEvent.BUTTON1) {
			guiHandler.pressMouse(e.getPoint().x, e.getPoint().y);
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * Invoked when a mouse button has been released on a component.
	 * 
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		if (mouseButton == MouseEvent.BUTTON1) {
			if (dragging) {
				dragging = false;
				guiHandler.endDragMouse(e.getPoint().x, e.getPoint().y);
			}
		}
		mouseButton = 0;
	}
}
