/*
 * CustomFileChooser.java       Jan 19, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc.io;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import java.io.File;

/**
 * A class that allows choosing a file. It performs some checks after the user
 * clicks the approve button.
 * 
 * @author Brian Mulanda (<a
 *         href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see IO
 * 
 */

public class CustomFileChooser extends JFileChooser {
	static final long serialVersionUID = 1;

	public static final int TYPE_OPEN = 0;
	public static final int TYPE_SAVE = 1;
	public static final int TYPE_SAVEAS = 2;

	private int type;

	public CustomFileChooser() {
		this(null);
	}

	public CustomFileChooser(String currentDirectory) {
		super(currentDirectory);
		this.type = TYPE_OPEN;
	}

	public void setType(int type) {
		this.type = type;
	}

	public int getType() {
		return type;
	}

	public void approveSelection() {
		File f = getSelectedFile();
		boolean approve = true;
		if (getType() == TYPE_OPEN && !f.exists()) {
			JOptionPane.showMessageDialog(null, "File " + f.getAbsolutePath()
					+ " does not exist", getDialogTitle(),
					JOptionPane.ERROR_MESSAGE);
			approve = false;
		} else if (getType() == TYPE_SAVE || getType() == TYPE_SAVEAS) {
			if (f.exists()) {
				Object[] options = { "Continue", "Cancel" };
				int option = JOptionPane.showOptionDialog(null,
						"This will overwrite file " + f.getAbsolutePath(),
						"Warning", JOptionPane.DEFAULT_OPTION,
						JOptionPane.WARNING_MESSAGE, null, options, options[1]);
				if (option != 0) {
					approve = false;
				}
			}
		}

		if (approve) {
			super.approveSelection();
		}
	}
}
