package edu.ksu.cis.ndpc.rules;

/**
 * @author Brian Mulanda
 * 
 */
public abstract class Rule {
	public static final int INV = -1;
	public static final int ASSUMPTION = 0;
	public static final int PREMISE = 1;
	public static final int DIMP_ELIM1 = 2;
	public static final int DIMP_ELIM2 = 3;
	public static final int COPY = 4;
	public static final int DIMP_INTRO = 5;
	public static final int IMP_INTRO = 6;
	public static final int IMP_ELIM = 7;
	public static final int SYM = 8;
	public static final int PBC = 9;
	public static final int LEM = 10;
	public static final int OR_INTRO1 = 11;
	public static final int OR_INTRO2 = 12;
	public static final int DNEG_INTRO = 13;
	public static final int DNEG_ELIM = 14;
	public static final int AND_ELIM1 = 15;
	public static final int AND_ELIM2 = 16;
	public static final int AND_INTRO = 17;
	public static final int MT = 18;
	public static final int OR_ELIM = 19;
	public static final int NEG_INTRO = 20;
	public static final int NEG_ELIM = 21;
	public static final int FORALL_ELIM = 22;
	public static final int EXIST_ELIM = 23;
	public static final int FORALL_INTRO = 24;
	public static final int EXIST_INTRO = 25;
	public static final int EQUAL_INTRO = 26;
	public static final int EQUAL_ELIM = 27;
	public static final int BOTTOM_ELIM = 28;

	private static Rule[] rules = new Rule[29];

	public abstract int getType();

	public abstract void check(RuleParameter[] params) throws RuleException;

	public static Rule getRule(int type) {
		Rule rule = null;

		if (type > -1 && type <= 29) {
			if (rules[type] != null) {
				rule = rules[type];
			} else {
				if (type == ASSUMPTION)
					rule = new RuleAssumption();
				else if (type == PREMISE)
					rule = new RulePremise();
				else if (type == DIMP_ELIM1)
					rule = new RuleDbImpElim1();
				else if (type == DIMP_ELIM2)
					rule = new RuleDbImpElim2();
				else if (type == COPY)
					rule = new RuleCOPY();
				else if (type == DIMP_INTRO)
					rule = new RuleDbImpIntro();
				else if (type == IMP_INTRO)
					rule = new RuleImpIntro();
				else if (type == IMP_ELIM)
					rule = new RuleImpElim();
				else if (type == SYM)
					rule = new RuleSYM();
				else if (type == PBC)
					rule = new RulePBC();
				else if (type == LEM)
					rule = new RuleLEM();
				else if (type == OR_INTRO1)
					rule = new RuleOrIntro1();
				else if (type == OR_INTRO2)
					rule = new RuleOrIntro2();
				else if (type == DNEG_INTRO)
					rule = new RuleDbNegIntro();
				else if (type == DNEG_ELIM)
					rule = new RuleDbNegElim();
				else if (type == AND_ELIM1)
					rule = new RuleAndElim1();
				else if (type == AND_ELIM2)
					rule = new RuleAndElim2();
				else if (type == AND_INTRO)
					rule = new RuleAndIntro();
				else if (type == MT)
					rule = new RuleMT();
				else if (type == OR_ELIM)
					rule = new RuleOrElim();
				else if (type == NEG_INTRO)
					rule = new RuleNegIntro();
				else if (type == NEG_ELIM)
					rule = new RuleNegElim();
				else if (type == FORALL_ELIM)
					rule = new RuleForallElim();
				else if (type == EXIST_ELIM)
					rule = new RuleExistsElim();
				else if (type == FORALL_INTRO)
					rule = new RuleForallIntro();
				else if (type == EXIST_INTRO)
					rule = new RuleExistsIntro();
				else if (type == EQUAL_INTRO)
					rule = new RuleEqualsIntro();
				else if (type == EQUAL_ELIM)
					rule = new RuleEqualsElim();
				else if (type == BOTTOM_ELIM)
					rule = new RuleBottomElim();

				rules[type] = rule;
			}

		}

		return rule;
	}

	public static String getDescription(int type) {
		return getName(type) + " (" + getSymbol(type) + ")";
	}

	public static int identify(String symbol) {
		int type = INV;
		if (symbol != null) {
			symbol = symbol.trim().toLowerCase();
			if (symbol.equals("assumption")) {
				type = ASSUMPTION;
			} else if (symbol.equals("premise")) {
				type = PREMISE;
			} else if (symbol.equals("<->e1")) {
				type = DIMP_ELIM1;
			} else if (symbol.equals("<->e2")) {
				type = DIMP_ELIM2;
			} else if (symbol.equals("copy")) {
				type = COPY;
			} else if (symbol.equals("<->i")) {
				type = DIMP_INTRO;
			} else if (symbol.equals("->i")) {
				type = IMP_INTRO;
			} else if (symbol.equals("->e")) {
				type = IMP_ELIM;
			} else if (symbol.equals("sym")) {
				type = SYM;
			} else if (symbol.equals("pbc")) {
				type = PBC;
			} else if (symbol.equals("lem")) {
				type = LEM;
			} else if (symbol.equals("vi1")) {
				type = OR_INTRO1;
			} else if (symbol.equals("vi2")) {
				type = OR_INTRO2;
			} else if (symbol.equals("~~i")) {
				type = DNEG_INTRO;
			} else if (symbol.equals("~~e")) {
				type = DNEG_ELIM;
			} else if (symbol.equals("^e1")) {
				type = AND_ELIM1;
			} else if (symbol.equals("^e2")) {
				type = AND_ELIM2;
			} else if (symbol.equals("^i")) {
				type = AND_INTRO;
			} else if (symbol.equals("mt")) {
				type = MT;
			} else if (symbol.equals("ve")) {
				type = OR_ELIM;
			} else if (symbol.equals("~i")) {
				type = NEG_INTRO;
			} else if (symbol.equals("~e")) {
				type = NEG_ELIM;
			} else if (symbol.equals("ae")) {
				type = FORALL_ELIM;
			} else if (symbol.equals("ee")) {
				type = EXIST_ELIM;
			} else if (symbol.equals("ai")) {
				type = FORALL_INTRO;
			} else if (symbol.equals("ei")) {
				type = EXIST_INTRO;
			} else if (symbol.equals("=e")) {
				type = EQUAL_ELIM;
			} else if (symbol.equals("=i")) {
				type = EQUAL_INTRO;
			} else if (symbol.equals("_|_e")) {
				type = BOTTOM_ELIM;
			}
		}
		return type;
	}

	public static String getName(int type) {
		String name = "Unknown";
		if (type == ASSUMPTION)
			name = "assumption";
		else if (type == PREMISE)
			name = "premise";
		else if (type == DIMP_ELIM1)
			name = "DbImp-Elim1";
		else if (type == DIMP_ELIM2)
			name = "DbImp-Elim2";
		else if (type == COPY)
			name = "copy";
		else if (type == DIMP_INTRO)
			name = "DbImp-Intro";
		else if (type == IMP_INTRO)
			name = "Imp-Intro";
		else if (type == IMP_ELIM)
			name = "Imp-Elim";
		else if (type == SYM)
			name = "Sym";
		else if (type == PBC)
			name = "Pbc";
		else if (type == LEM)
			name = "Lem";
		else if (type == OR_INTRO1)
			name = "Or-Intro1";
		else if (type == OR_INTRO2)
			name = "Or-Intro2";
		else if (type == DNEG_INTRO)
			name = "DbNeg-Intro";
		else if (type == DNEG_ELIM)
			name = "DbNeg-Elim";
		else if (type == AND_ELIM1)
			name = "And-Elim1";
		else if (type == AND_ELIM2)
			name = "And-Elim2";
		else if (type == AND_INTRO)
			name = "And-Intro";
		else if (type == MT)
			name = "MT";
		else if (type == OR_ELIM)
			name = "Or-Elim";
		else if (type == NEG_INTRO)
			name = "Neg-Intro";
		else if (type == NEG_ELIM)
			name = "Neg-Elim";
		else if (type == FORALL_ELIM)
			name = "Forall-Elim";
		else if (type == EXIST_ELIM)
			name = "Exist-Elim";
		else if (type == FORALL_INTRO)
			name = "Forall-Intro";
		else if (type == EXIST_INTRO)
			name = "Exist-Intro";
		else if (type == EQUAL_INTRO)
			name = "Equal-Intro";
		else if (type == EQUAL_ELIM)
			name = "Equal-Elim";
		else if (type == BOTTOM_ELIM)
			name = "Bottom-Elim";

		return name;
	}

	public static String getSymbol(int type) {
		String sym = "";
		if (type == ASSUMPTION)
			sym = "assumption";
		else if (type == PREMISE)
			sym = "premise";
		else if (type == COPY)
			sym = "Copy";
		else if (type == SYM)
			sym = "Sym";
		else if (type == PBC)
			sym = "Pbc";
		else if (type == LEM)
			sym = "Lem";
		else if (type == DIMP_ELIM1)
			sym = "<->e1";
		else if (type == DIMP_ELIM2)
			sym = "<->e2";
		else if (type == DIMP_INTRO)
			sym = "<->i";
		else if (type == IMP_INTRO)
			sym = "->i";
		else if (type == IMP_ELIM)
			sym = "->e";
		else if (type == OR_INTRO1)
			sym = "vi1";
		else if (type == OR_INTRO2)
			sym = "vi2";
		else if (type == OR_ELIM)
			sym = "ve";
		else if (type == DNEG_INTRO)
			sym = "~~i";
		else if (type == DNEG_ELIM)
			sym = "~~e";
		else if (type == NEG_INTRO)
			sym = "~i";
		else if (type == NEG_ELIM)
			sym = "~e";
		else if (type == AND_INTRO)
			sym = "^i";
		else if (type == AND_ELIM1)
			sym = "^e1";
		else if (type == AND_ELIM2)
			sym = "^e2";
		else if (type == MT)
			sym = "MT";
		else if (type == FORALL_ELIM)
			sym = "Ae";
		else if (type == EXIST_ELIM)
			sym = "Ee";
		else if (type == FORALL_INTRO)
			sym = "Ai";
		else if (type == EXIST_INTRO)
			sym = "Ei";
		else if (type == EQUAL_INTRO)
			sym = "=i";
		else if (type == EQUAL_ELIM)
			sym = "=e";
		else if (type == BOTTOM_ELIM)
			sym = "_|_e";

		return sym;
	}
}