package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleAndElim1 extends Rule {
	private static String name = getDescription(Rule.AND_ELIM1);

	public RuleAndElim1() {
		super();
	}

	public int getType() {
		return Rule.AND_ELIM1;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires a formula");
		} else if (!((params[0] instanceof ProofStatement) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires a formula");
		}

		ProofStatement premise = (ProofStatement) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = premise.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_CONJ) {
			throw new RuleException("Formula " + premise
					+ " must have '^' as its main connective");
		} else {
			ParseTree leftSubtree = premise.getFormula().getParseTree()
					.getLeftSubtree();
			if (!leftSubtree.equals(conclusion.getFormula().getParseTree())) {
				throw new RuleException("Formula " + conclusion
						+ " must match left "
						+ "side of the main connective '^' in formula "
						+ premise);
			}
		}
	}
}