package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleAndIntro extends Rule {
	private static String name = getDescription(Rule.AND_INTRO);

	public RuleAndIntro() {
		super();
	}

	public int getType() {
		return Rule.AND_INTRO;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 3 || params[0] == null
				|| params[1] == null || params[2] == null) {
			throw new RuleException("Rule requires two formulas");
		} else if (!((params[0] instanceof ProofStatement)
				&& (params[1] instanceof ProofStatement) && (params[2] instanceof ProofStatement))) {
			throw new RuleException("Rule requires two formulas");
		}

		ProofStatement premise1 = (ProofStatement) params[0];
		ProofStatement premise2 = (ProofStatement) params[1];
		ProofStatement conclusion = (ProofStatement) params[2];

		Symbol root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_CONJ) {
			throw new RuleException("Formula " + conclusion
					+ " must have '^' as its main connective");
		} else {
			ParseTree rightSubtree = conclusion.getFormula().getParseTree()
					.getRightSubtree();
			ParseTree leftSubtree = conclusion.getFormula().getParseTree()
					.getLeftSubtree();
			if (!leftSubtree.equals(premise1.getFormula().getParseTree())) {
				throw new RuleException(
						"Left side of the main connective '^' in formula "
								+ conclusion + " must match formula "
								+ premise1);
			} else if (!rightSubtree.equals(premise2.getFormula()
					.getParseTree())) {
				throw new RuleException(
						"Right side of the main connective '^' in formula "
								+ conclusion + " must match formula "
								+ premise2);
			}
		}
	}
}