package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.BoxedStatements;
import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleDbImpIntro extends Rule {
	private static String name = getDescription(Rule.DIMP_INTRO);

	public RuleDbImpIntro() {
		super();
	}

	public int getType() {
		return Rule.DIMP_INTRO;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 3 || params[0] == null
				|| params[1] == null || params[2] == null) {
			throw new RuleException("Rule requires two sets of boxed formulas");
		} else if (!((params[0] instanceof BoxedStatements)
				&& (params[1] instanceof BoxedStatements) && (params[2] instanceof ProofStatement))) {
			throw new RuleException("Rule requires two sets of boxed formulas");
		}

		BoxedStatements premise1 = (BoxedStatements) params[0];
		BoxedStatements premise2 = (BoxedStatements) params[1];
		ProofStatement conclusion = (ProofStatement) params[2];

		Symbol root = conclusion.getFormula().getParseTree().getRootNode();

		ProofStatement box1FstForm = premise1.getFirst();
		ProofStatement box1LstForm = premise1.getLast();
		ProofStatement box2FstForm = premise2.getFirst();
		ProofStatement box2LstForm = premise2.getLast();

		ParseTree cLeftSubtree = conclusion.getFormula().getParseTree()
				.getLeftSubtree();
		ParseTree cRightSubtree = conclusion.getFormula().getParseTree()
				.getRightSubtree();

		ParseTree box1FstTree = box1FstForm.getFormula().getParseTree();
		ParseTree box1LstTree = box1LstForm.getFormula().getParseTree();
		ParseTree box2FstTree = box2FstForm.getFormula().getParseTree();
		ParseTree box2LstTree = box2LstForm.getFormula().getParseTree();

		if (!box2FstTree.equals(box1LstTree)) {
			throw new RuleException("The first formula in " + box2FstForm
					+ " in the" + "box: " + premise2.getRange()
					+ " must be the same as the last formula " + box1FstForm
					+ " in the box: " + premise1.getRange());
		} else if (!box2LstTree.equals(box1FstTree)) {
			throw new RuleException("The last formula in " + box2FstForm
					+ " in the" + "box: " + premise2.getRange()
					+ " must be the same as the first formula " + box1FstForm
					+ " in the box: " + premise1.getRange());
		} else {
			if (root.getType() != LangSymbols.S_DIMPL) {
				throw new RuleException("Formula " + conclusion
						+ " must have '<->' as its main connective");
			} else if (!cLeftSubtree.equals(box1FstTree)) {
				throw new RuleException(
						"The formula on the left side of the main connective "
								+ "'<->' in formula " + conclusion
								+ " must be the same as the "
								+ "first formula " + box1FstForm
								+ " in the box: " + premise1.getRange());
			} else if (!cRightSubtree.equals(box1LstTree)) {
				throw new RuleException(
						"The formula on the right side of the main connective "
								+ "'<->' in formula " + conclusion
								+ " must be the same as the "
								+ "first formula " + box2FstForm
								+ " in the box: " + premise2.getRange());
			} else {
				// OK
			}
		}
	}
}