package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleDbNegIntro extends Rule {
	private static String name = getDescription(Rule.DNEG_INTRO);

	public RuleDbNegIntro() {
		super();
	}

	public int getType() {
		return Rule.DNEG_INTRO;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires a formula");
		} else if (!((params[0] instanceof ProofStatement) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires a formula");
		}

		ProofStatement premise = (ProofStatement) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = null;
		root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("Formula " + conclusion
					+ " must negated at least twice (begin with ~~");
		}

		root = conclusion.getFormula().getParseTree().getSubtree()
				.getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("Formula " + conclusion
					+ " must negated at least twice (begin with ~~)");
		}

		ParseTree subtree = conclusion.getFormula().getParseTree().getSubtree()
				.getSubtree();
		if (!subtree.equals(premise.getFormula().getParseTree())) {
			throw new RuleException(
					"Formula "
							+ conclusion
							+ " must be such that if you "
							+ "remove the first two negations ('~~') the resulting formula "
							+ "is the same as formula " + premise);
		}
	}
}