package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.BoxedStatements;
import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofChecker;
import edu.ksu.cis.ndpc.ProofCheckerException;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleForallIntro extends Rule {
	private static String name = getDescription(Rule.FORALL_INTRO);

	public RuleForallIntro() {
		super();
	}

	public int getType() {
		return Rule.FORALL_INTRO;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires one set of boxed formulas");
		} else if (!((params[0] instanceof BoxedStatements) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires one set of boxed formulas");
		}

		BoxedStatements premise = (BoxedStatements) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		ParseTree formulaTree = premise.getLast().getFormula().getParseTree();
		ParseTree conclusionTree = conclusion.getFormula().getParseTree();
		ParseTree variable = null;

		Symbol freshVar = premise.getFirst().getVariable();
		if (freshVar != null && freshVar.getValue() != null) {
			variable = new ParseTree(freshVar);
		} else {
			throw new RuleException(
					"You must introduce a fresh variable in line "
							+ premise.getFirst().getLineNo() + ".");
		}

		if (premise.getFirst().getFormula() != null
				&& premise.getFirst().getRule() != null
				&& premise.getFirst().getRule().type == Rule.ASSUMPTION) {
			throw new RuleException(name + ": The formula on line "
					+ premise.getFirst().getLineNo()
					+ " cannot be an assumption, "
					+ "because it is on the same line that introduces a fresh "
					+ "variable in a forall introduction box.");
		}

		if (conclusionTree.getRootNode().getType() != LangSymbols.S_FORALL) {
			throw new RuleException("Formula " + conclusion
					+ " must be universally " + "quantified ");
		}

		Symbol var = new Symbol(conclusionTree.getRootNode().getValue()
				.substring(1), LangSymbols.S_VAR);
		ParseTree termX = new ParseTree(var);
		ParseTree termT = variable;

		ParseTree testConclTree = ProofChecker.substitute(
				(ParseTree) conclusionTree.getSubtree().clone(), termX, termT);

		if (!testConclTree.equals(formulaTree)) {
			String formula = conclusion.getFormula().getFormulaString()
					.substring(0, conclusionTree.getRootNode().getLength())
					.trim();

			throw new RuleException("Incorrect substitution. From "
					+ conclusion + ", you should be able to obtain "
					+ premise.getLast() + " by first " + "removing " + formula
					+ ", and then replacing all free occurrences " + "of "
					+ var + " with " + variable + ".");
		}
	}
}
