package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleLEM extends Rule {
	private static String name = getDescription(Rule.LEM);

	public RuleLEM() {
		super();
	}

	public int getType() {
		return Rule.LEM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		/*
		 * if ( params == null || params.length != 1 || params[0] == null ) {
		 * throw new RuleException("Rule requires one formula"); } else if ( !
		 * (params[0] instanceof ProofStatement) ) { throw new
		 * RuleException("Rule requires one formula"); }
		 */

		ProofStatement conclusion = (ProofStatement) params[0];

		Symbol root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_DISJ) {
			throw new RuleException("Formula " + conclusion + " must "
					+ "have 'v' as its main connective: formula_f v ~formula_f");
		}

		ParseTree leftSubtree = conclusion.getFormula().getParseTree()
				.getLeftSubtree();
		ParseTree rightSubtree = conclusion.getFormula().getParseTree()
				.getRightSubtree();

		root = rightSubtree.getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException(
					"The right side of the main connective 'v' in formula "
							+ conclusion
							+ " must be negated: formula_f v ~formula_f");
		}

		if (!rightSubtree.getSubtree().equals(leftSubtree)) {
			throw new RuleException(
					"Formula "
							+ conclusion
							+ " must be such that if you remove "
							+ "the first negation from the left side of the main connective 'v',"
							+ "the resulting formula should be the same as the left side of "
							+ "the connective: formula_f v ~formula_f");
		}
	}
}