package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.BoxedStatements;
import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleNegIntro extends Rule {
	private static String name = getDescription(Rule.NEG_INTRO);

	public RuleNegIntro() {
		super();
	}

	public int getType() {
		return Rule.NEG_INTRO;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires one set of boxed formulas");
		} else if (!((params[0] instanceof BoxedStatements) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires one set of boxed formulas");
		}

		BoxedStatements premise = (BoxedStatements) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("Formula " + conclusion
					+ " must be negated "
					+ "(begin with a '~') like this: ~formula or ~(formula)");
		}

		ProofStatement lastFormula = premise.getLast();
		root = lastFormula.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_BOTTOM) {
			throw new RuleException("The last formula in box: "
					+ premise.getRange() + "must be bottom '_|_'");
		}

		ProofStatement firstFormula = premise.getFirst();
		if (firstFormula.getRule().type != Rule.ASSUMPTION) {
			throw new RuleException("The first formula " + firstFormula + " "
					+ "in box " + premise.getRange() + " must be an "
					+ "assumption");
		}

		ParseTree cSubtree = conclusion.getFormula().getParseTree()
				.getSubtree();

		if (!cSubtree.equals(firstFormula.getFormula().getParseTree())) {
			throw new RuleException("The negated formula in " + conclusion
					+ " must be " + "the same as the first formula "
					+ firstFormula + " in the " + "box: " + premise.getRange());
		}
	}
}