package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleOrIntro2 extends Rule {
	private static String name = getDescription(Rule.OR_INTRO2);

	public RuleOrIntro2() {
		super();
	}

	public int getType() {
		return Rule.OR_INTRO2;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires a formula");
		} else if (!((params[0] instanceof ProofStatement) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires a formula");
		}

		ProofStatement premise = (ProofStatement) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_DISJ) {
			throw new RuleException("Formula " + conclusion
					+ " must have 'v' as its main connective");
		} else {
			ParseTree rightSubtree = conclusion.getFormula().getParseTree()
					.getRightSubtree();
			if (!rightSubtree.equals(premise.getFormula().getParseTree())) {
				throw new RuleException(
						"The right side of the main connective 'v' in formula"
								+ conclusion + " must match formula " + premise);
			}
		}
	}
}